﻿// ----------------------------------------------------------------------
// <copyright file="AudioCapture.cs" company="Bryan Biedenkapp">
//     Copyright (c) 2012 Bryan Biedenkapp., All Rights Reserved.
// </copyright>
// ----------------------------------------------------------------------
/**
 * MDCTool
 * INTERNAL/PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Author: Bryan Biedenkapp <gatekeep@jmp.cx>
 * 
 * $created guid: 7d9b8ab8-1cbc-4793-a3cd-d8c49d5e5f89 2012/1/11$
 */
using System;
using System.IO;
using System.Collections.Generic;
using System.Threading;

using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

namespace MDCTool
{
    /**
     * Delegates
     */
    /// <summary>
    /// Voice Content Sent Notification Delegate
    /// </summary>
    /// <param name="data">Raw Wave Data</param>
    public delegate int ContentNotifierDelegate(byte[] data);

    /// <summary>
    /// Helper class to aid in capturing voice audio.
    /// </summary>
    public class AudioCapture : IDisposable
    {
        /**
         * Fields
         */
        private WaveFormat waveFormat;                  // Referenced WaveForamt Instance

        private Thread thread;                          //

        private CaptureBufferDescription captureBufferDesc;
        private CaptureBuffer captureBuffer;            // DirectSound Capture Buffer
        private int captureHalfBuffer;                  //
        private int capturePos = 0;                     // Capture Position
        private int readPos = 0;                        // Read Position

        private byte[] byteData;                        // Capture Raw short Array

        private AutoResetEvent bufferNotification;

        private bool rxAudio;                           // Is the capture thread running?
        private bool disposed;

        /**
         * Events
         */
        /// <summary>
        /// Occurs when the voice content is being sent.
        /// </summary>
        public event ContentNotifierDelegate ContentNotification;

        // Methods
        /// <summary>
        /// Instantiate voice capture handler.
        /// </summary>
        public AudioCapture(Capture audCapture, WaveFormat format)
        {
            this.bufferNotification = new AutoResetEvent(false);
            this.waveFormat = format;

            thread = new Thread(new ThreadStart(ThreadProc));
            thread.Name = "AudioCapture<ThreadProc>";
            thread.Priority = ThreadPriority.AboveNormal;
            rxAudio = true;
            thread.Start();

            int _dwNotifySize = format.AverageBytesPerSecond / 8;
            _dwNotifySize -= _dwNotifySize % format.BlockAlign;

            // initialize capture buffer description
            captureBufferDesc = new CaptureBufferDescription();
            captureBufferDesc.BufferBytes = MainWindow.NUM_BUFFERS * _dwNotifySize;
            captureBufferDesc.Format = waveFormat;

            // intialize capture memory
            captureHalfBuffer = MainWindow.NUM_BUFFERS * _dwNotifySize / 2;
            byteData = new byte[captureHalfBuffer];

            // initialize capture buffer
            captureBuffer = new CaptureBuffer(captureBufferDesc, audCapture);

            // create notifications
            Notify n = new Notify(captureBuffer);
            BufferPositionNotify[] bpns = new BufferPositionNotify[2];

            // midway notification
            bpns[0] = new BufferPositionNotify();
            bpns[0].Offset = (captureBufferDesc.BufferBytes / 2) - 1;
            bpns[0].EventNotifyHandle = bufferNotification.SafeWaitHandle.DangerousGetHandle();

            // end notification
            bpns[1] = new BufferPositionNotify();
            bpns[1].Offset = captureBufferDesc.BufferBytes - 1;
            bpns[1].EventNotifyHandle = bufferNotification.SafeWaitHandle.DangerousGetHandle();

            n.SetNotificationPositions(bpns);
        }

        /// <summary>
        /// Start audio capture.
        /// </summary>
        public void Start()
        {
            // start capturing audio
            captureBuffer.Start(true);
        }

        /// <summary>
        /// Stop audio capture.
        /// </summary>
        public void Stop()
        {
            // stop capturing audio
            captureBuffer.Stop();
        }

        /// <summary>
        /// Implement IDisposable.
        /// Do not make this method virtual.
        /// A derived class should not be able to override this method.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            // This object will be cleaned up by the Dispose method.
            // Therefore, you should call GC.SupressFinalize to
            // take this object off the finalization queue 
            // and prevent finalization code for this object
            // from executing a second time.
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Dispose(bool disposing) executes in two distinct scenarios.
        /// If disposing equals true, the method has been called directly
        /// or indirectly by a user's code. Managed and unmanaged resources
        /// can be disposed.
        /// If disposing equals false, the method has been called by the 
        /// runtime from inside the finalizer and you should not reference 
        /// other objects. Only unmanaged resources can be disposed.
        /// </summary>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!this.disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    rxAudio = false;
                    if (bufferNotification != null)
                        bufferNotification.Set();

                    // if we're capturing stop capturing
                    if (captureBuffer.Capturing)
                    {
                        // stop capturing
                        captureBuffer.Stop();
                    }

                    // stop capturing
                    if (captureBuffer != null)
                    {
                        if (captureBuffer.Capturing)
                            captureBuffer.Stop();

                        captureBuffer.Dispose();
                        captureBuffer = null;
                    }

                    this.captureHalfBuffer = 0;
                    this.byteData = null;
                }
            }
            disposed = true;
        }

        /// <summary>
        /// Internal function to reset the data stream.
        /// </summary>
        private void ResetStream()
        {
            byteData = new byte[captureHalfBuffer];
        }

        /// <summary>
        /// Internal thread procedure.
        /// </summary>
        private void ThreadProc()
        {
            while (rxAudio)
            {
                // ensure the capturebuffer is instantiated and not disposed
                if (captureBuffer != null && captureBuffer.Disposed != true)
                {
                    // ensure we're capturing
                    if (captureBuffer.Capturing)
                    {
                        // wait for a notification from direct sound
                        bufferNotification.WaitOne();

                        captureBuffer.GetCurrentPosition(out capturePos, out readPos);
                        if (capturePos > readPos)
                        {
                            // recording in the second half of the buffer
                            Array data = captureBuffer.Read(0, typeof(byte), LockFlag.None, captureHalfBuffer);
                            Array.Copy(data, byteData, data.Length);
                        }
                        else
                        {
                            // recording to front half of buffer
                            Array data = captureBuffer.Read(captureHalfBuffer, typeof(byte), LockFlag.None, captureHalfBuffer);
                            Array.Copy(data, byteData, data.Length);
                        }

                        // fire content notification event
                        if (ContentNotification != null)
                            ContentNotification(byteData);

                        // reset stream
                        ResetStream();
                    }
                }
            }
        }
    } // public class AudioCapture : IDisposable
} // namespace MDCTool
