﻿// ----------------------------------------------------------------------
// <copyright file="MainWindow.cs" company="Bryan Biedenkapp">
//     Copyright (c) 2012 Bryan Biedenkapp., All Rights Reserved.
// </copyright>
// ----------------------------------------------------------------------
/**
 * MDCTool
 * INTERNAL/PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Author: Bryan Biedenkapp <gatekeep@jmp.cx>
 * 
 * $created guid: 56891fe7-90dd-4940-8ceb-903c7ea4cbda 2012/1/11$
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.DirectX.DirectSound;

using MDC1200;

using MDCTool.Wave;

namespace MDCTool
{
    using MDCDecoder = MDC1200.Decoder;
    using MDCEncoder = MDC1200.Encoder;

    /// <summary>
    /// This class serves as the main interface for the MDCTool application.
    /// </summary>
    public partial class MainWindow : Form
    {
        /**
         * Constants
         */
        /// <summary>
        /// Audio Sample Rate
        /// </summary>
        public const int SAMPLE_RATE = 22050;
        /// <summary>
        /// Bits Per Sample (Bit Rate)
        /// </summary>
        public const int BITS_PER_SAMPLE = 8;

        public const int NUM_BUFFERS = 16;
        public const int BUFFER_SIZE = 2048;

        public const int MAX_PREAMBLES_ALLOWED = 10;

        /**
         * Fields
         */
        private ushort myID;                            // MDCTool's PTT ID
        private ushort targetID;                        // Target Radio ID

        private byte debugFirstOp;
        private byte debugFirstArg;
        private ushort debugFirstUnitID;
        private byte debugSecondOp;
        private byte debugSecondArg;
        private ushort debugSecondUnitID;

        private bool isCapturing;                       //
        private AudioCapture audCap;                    //

        private WaveFormat waveFormat;                  //

        private BufferDescription playbackBufDesc;      //
        private SecondaryBuffer playbackBuffer;         // DirectSound Playback Buffer

        private delegate void __UpdateListBox(string str);

        /**
         * Properties
         */
        /// <summary>
        /// MDC1200 Decoder
        /// </summary>
        public MDCDecoder Decoder
        {
            get;
            private set;
        }

        /// <summary>
        /// MDC1200 Encoder
        /// </summary>
        public MDCEncoder Encoder
        {
            get;
            private set;
        }

        /// <summary>
        /// Direct/X Audio Device
        /// </summary>
        public Device AudioDevice
        {
            get;
            private set;
        }

        /// <summary>
        /// Direct/X Audio Capture Device
        /// </summary>
        public Capture CaptureDevice
        {
            get;
            set;
        }

        /// <summary>
        /// Output Device Collection
        /// </summary>
        public DevicesCollection OutputDevices
        {
            get;
            private set;
        }

        /// <summary>
        /// Capture Device Collection
        /// </summary>
        public CaptureDevicesCollection CaptureDevices
        {
            get;
            private set;
        }

        /**
         * Methods
         */
        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            Decoder = new MDCDecoder(SAMPLE_RATE);
            Encoder = new MDCEncoder(SAMPLE_RATE);

            this.isCapturing = false;

            this.myID = 0x0001;
            this.targetID = 0x0001;

            this.Encoder.NumberOfPreambles = 3;

            this.debugFirstOp = 0x00;
            this.debugFirstArg = 0x00;
            this.debugFirstUnitID = 0x00;
            this.debugSecondOp = 0x00;
            this.debugSecondArg = 0x00;
            this.debugSecondUnitID = 0x00;

            this.MaximumSize = this.MinimumSize = this.Size = new Size(this.Width, 340);
            this.firstGroupBox.Visible = false;
            this.secondGroupBox.Visible = false;

            this.FormClosed += new FormClosedEventHandler(MainWindow_FormClosed);

            try
            {
                InitializeDirectX();
            }
            catch (Exception e)
            {
                StackTrace.Write("There was an error while trying to initialize Direct/X. This error is fatal and we cannot" +
                    " continue.", e);
                throw e;
            }

            try
            {
                // initialize audio capture
                audCap = new AudioCapture(CaptureDevice, waveFormat);
                audCap.ContentNotification += Decoder.ProcessSamples;
                this.startCapturingToolStripMenuItem.Enabled = true;
            }
            catch (Exception e)
            {
                StackTrace.Write("There was an error while trying to setup audio capturing! Audio capturing will be" +
                    " disabled for this session.", e);
            }

            // hook decoder callback to display any decoded packets
            Decoder.DecoderCallback += new MDCDecoderCallback(Decoder_DecoderCallback);
            Decoder.DecoderFailedCallback += new MDCDecoderFailedCallback(Decoder_DecoderFailedCallback);
        }

        /// <summary>
        /// Occurs when the MDC-1200 decoder failes to decode a packet.
        /// </summary>
        /// <param name="frameCount"></param>
        void Decoder_DecoderFailedCallback(int frameCount)
        {
            string timeString = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToLongTimeString();
            recievedPackets.Invoke(new __UpdateListBox(delegate(string str)
            {
                recievedPackets.Items.Insert(0, timeString + " CRC Mismatch! Bad MDC Frame " + frameCount);
            }), new object[] { null });
        }

        /// <summary>
        /// Occurs when the MDC-1200 decoder, decodes a packet.
        /// </summary>
        private void Decoder_DecoderCallback(int frameCount, MDCPacket first, MDCPacket second)
        {
            string timeString = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToLongTimeString();
            recievedPackets.Invoke(new __UpdateListBox(delegate(string str)
                {
                    recievedPackets.Items.Insert(0, timeString + " " + str);
                }), new object[] { Decoder.ToString(first) });

            if (frameCount == 2)
                recievedPackets.Invoke(new __UpdateListBox(delegate(string str)
                    {
                        recievedPackets.Items.Insert(0, timeString + " " + str);
                    }), new object[] { Decoder.ToString(second) });
        }

        /// <summary>
        /// Internal function to initialize DirectSound.
        /// </summary>
        private void InitializeDirectX()
        {
            // enumerate output devices
            OutputDevices = new DevicesCollection();
            DeviceInformation deviceInfo = OutputDevices[0];

            // initialize the audio device
            AudioDevice = new Device(deviceInfo.DriverGuid);

            // set cooperation level of the audio device
            if (AudioDevice != null)
                AudioDevice.SetCooperativeLevel(this.Handle, CooperativeLevel.Normal);

            // enumerate capture devices
            CaptureDevices = new CaptureDevicesCollection();
            deviceInfo = CaptureDevices[0];

            // initialize the capture device
            CaptureDevice = new Capture(deviceInfo.DriverGuid);

            // create format type
            WaveFormat format = new WaveFormat
            {
                SamplesPerSecond = SAMPLE_RATE,
                BitsPerSample = BITS_PER_SAMPLE,
                Channels = 1,
                FormatTag = WaveFormatTag.Pcm
            };

            format.BlockAlign = (short)(format.Channels * (format.BitsPerSample / 8));
            format.AverageBytesPerSecond = format.SamplesPerSecond * format.BlockAlign;

            int _dwNotifySize = format.AverageBytesPerSecond / 8;
            _dwNotifySize -= _dwNotifySize % format.BlockAlign;

            this.waveFormat = format;

            // initialize playback buffer description
            playbackBufDesc = new BufferDescription();
            playbackBufDesc.BufferBytes = NUM_BUFFERS * _dwNotifySize;
            playbackBufDesc.Format = waveFormat;

            // initialize playback buffer
            playbackBuffer = new SecondaryBuffer(playbackBufDesc, AudioDevice);
        }

        /// <summary>
        /// Occurs when the "About..." menu item is clicked.
        /// </summary>
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }

        /// <summary>
        /// Occurs when the main program dialog is closed.
        /// </summary>
        private void MainWindow_FormClosed(object sender, FormClosedEventArgs e)
        {
            // forward to the exit tool strip event handler...
            exitToolStripMenuItem_Click(sender, e);
        }

        /// <summary>
        /// Occurs when the "Exit" menu item is clicked.
        /// </summary>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Environment.Exit(0);
        }

        /// <summary>
        /// Occurs when the "Clear..." menu item is clicked in the Rx packets context menu.
        /// </summary>
        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            recievedPackets.Items.Clear();
        }

        /// <summary>
        /// Occurs when the "Start Capturing..." menu item is clicked.
        /// </summary>
        private void startCapturingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (isCapturing)
            {
                isCapturing = false;
                startCapturingToolStripMenuItem.Text = "Start Capturing...";
                audCap.Stop();
            }
            else
            {
                isCapturing = true;
                startCapturingToolStripMenuItem.Text = "Stop Capturing...";
                audCap.Start();
            }
        }

        /// <summary>
        /// Occurs when the "Open WAV..." menu item is clicked.
        /// </summary>
        private void openWAVToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // open a file
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "Wave Audio Files (*.wav)|*.wav|All files (*.*)|*.*";
            DialogResult ret = ofd.ShowDialog();

            if (ret == DialogResult.Cancel)
            {
                ofd.Dispose();
                return;
            }

            // open wav and decode
            WAVFile wavFile = new WAVFile();
            wavFile.Open(ofd.FileName, WAVFile.WAVFileMode.READ);

            // make sure we have the correct sample rate
            if (wavFile.SampleRateHz != SAMPLE_RATE)
            {
                MessageBox.Show("Cannot open " + ofd.FileName + "! The sample rate of the specified file is " + wavFile.SampleRateHz + "hz we can" +
                    " only process " + SAMPLE_RATE + "hz wave files!", "MDCTool Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                wavFile.Close();
                ofd.Dispose();
                return;
            }

            // create a buffer to contain the sample data from the wave file
            byte[] tmpBuf = new byte[wavFile.DataSizeBytes];
            for (int i = 0; i < wavFile.NumSamples; i++)
                tmpBuf[i] = wavFile.GetNextSampleAs8Bit();

            // decode MDC1200
            Decoder.ProcessSamples(tmpBuf);

            wavFile.Close();
            ofd.Dispose();
        }

        /// <summary>
        /// Occurs when the "Stream Encoded MDC-1200" menu item is clicked.
        /// </summary>
        private void streamEncodedMDC1200ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            streamEncodedMDC1200ToolStripMenuItem.Checked = true;
            saveEncodedMDC1200ToolStripMenuItem.Checked = false;
        }

        /// <summary>
        /// Occurs when the "Save Encoded MDC-1200" menu item is clicked.
        /// </summary>
        private void saveEncodedMDC1200ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            streamEncodedMDC1200ToolStripMenuItem.Checked = false;
            saveEncodedMDC1200ToolStripMenuItem.Checked = true;
        }

        /// <summary>
        /// Occurs when the "Only Decode" menu item is clicked.
        /// </summary>
        private void onlyDecodeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (onlyDecodeToolStripMenuItem.Checked)
            {
                onlyDecodeToolStripMenuItem.Checked = false;
                streamEncodedMDC1200ToolStripMenuItem.Enabled = true;
                saveEncodedMDC1200ToolStripMenuItem.Enabled = true;
            }
            else
            {
                onlyDecodeToolStripMenuItem.Checked = true;
                streamEncodedMDC1200ToolStripMenuItem.Enabled = false;
                saveEncodedMDC1200ToolStripMenuItem.Enabled = false;
            }
        }

        /// <summary>
        /// Internal function to invalidate the PTT ID input box.
        /// </summary>
        private void InvalidateMyID()
        {
            toolMDCID.ForeColor = Color.Red;
            myID = 0x0001;
        }

        /// <summary>
        /// Internal function to invalidate the Target ID input box.
        /// </summary>
        private void InvalidateTargetID()
        {
            targetMDCID.ForeColor = Color.Red;
            targetID = 0x0001;
        }

        /// <summary>
        /// Internal function to validate the PTT ID input box.
        /// </summary>
        private void ValidateMyID()
        {
            try
            {
                toolMDCID.ForeColor = Color.Black;
                myID = Convert.ToUInt16(toolMDCID.Text, 16);
            }
            catch (FormatException)
            {
                InvalidateMyID();
            }
        }

        /// <summary>
        /// Internal function to validate the Target ID input box.
        /// </summary>
        private void ValidateTargetID()
        {
            try
            {
                targetMDCID.ForeColor = Color.Black;
                targetID = Convert.ToUInt16(targetMDCID.Text, 16);
            }
            catch (FormatException)
            {
                InvalidateTargetID();
            }
        }

        /// <summary>
        /// Occurs when the text in the PTT ID box changes.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void toolMDCID_TextChanged(object sender, EventArgs e)
        {
            if (toolMDCID.Text.Length < 4)
                InvalidateMyID();
            else
            {
                char[] checkValues = toolMDCID.Text.ToCharArray();
                if ((checkValues[0] == 'F') || (checkValues[1] == 'F') || (checkValues[2] == 'F') || (checkValues[3] == 'F') ||
                    (checkValues[0] == 'E'))
                    InvalidateMyID();
                else
                    ValidateMyID();
            }
        }

        /// <summary>
        /// Occurs when the text in the Target ID box changes.
        /// </summary>
        private void targetMDCID_TextChanged(object sender, EventArgs e)
        {
            if (targetMDCID.Text.Length < 4)
                InvalidateTargetID();
            else
                ValidateTargetID();
        }

        /// <summary>
        /// Internal function to save the data generated by MDCTool.
        /// </summary>
        private void SaveGeneratedData()
        {
            byte[] tmpBuffer = new byte[BUFFER_SIZE * NUM_BUFFERS];
            int numOfSamples = 0;

            // copy samples into a temporary buffer
            numOfSamples += Encoder.GetSamples(ref tmpBuffer);

            if (!onlyDecodeToolStripMenuItem.Checked)
            {
                if (streamEncodedMDC1200ToolStripMenuItem.Checked)
                {
                    try
                    {
                        byte[] audioBuffer = new byte[numOfSamples];
                        for (int i = 0; i < numOfSamples; i++)
                            audioBuffer[i] = tmpBuffer[i];

                        playbackBuffer.Write(0, audioBuffer, LockFlag.None);
                        playbackBuffer.Play(0, BufferPlayFlags.Default);
                    }
                    catch (Exception e)
                    {
                        StackTrace.Write("There was an error while trying to play audio!", e);
                    }
                }
                else
                {
                    SaveFileDialog sfd = new SaveFileDialog();
                    sfd.Filter = "Wave Audio Files (*.wav)|*.wav|All files (*.*)|*.*";
                    DialogResult ret = sfd.ShowDialog();

                    if (ret == DialogResult.Cancel)
                    {
                        sfd.Dispose();
                        return;
                    }

                    // create wave file
                    WAVFile wavFile = new WAVFile();
                    wavFile.Create(sfd.FileName, false, SAMPLE_RATE, BITS_PER_SAMPLE);

                    // write data to wave file
                    for (int i = 0; i < numOfSamples; i++)
                        wavFile.AddSample_8bit(tmpBuffer[i]);

                    // write 4 sample pad
                    for (int i = 0; i < 4; i++)
                        wavFile.AddSample_8bit(0);

                    wavFile.Close();
                    sfd.Dispose();
                }
            }

            if (numOfSamples > 0)
            {
                // push data to decoder
                if (decodeAfterEncodingToolStripMenuItem.Checked)
                {
                    short[] decBuffer = new short[numOfSamples];
                    for (int i = 0; i < numOfSamples; i++)
                        decBuffer[i] = tmpBuffer[i];

                    Decoder.ProcessSamples(tmpBuffer);
                }
            }
        }

        /// <summary>
        /// Occurs when the "Tool PTT ID" button is pressed.
        /// </summary>
        private void txToolPTT_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = OpType.PTT_ID,
                Argument = ArgType.NO_ARG,
                UnitID = myID
            };
            Encoder.CreateSingle(pckt);
            SaveGeneratedData();
        }

        /// <summary>
        /// Occurs when the "Stun Target" button is pressed.
        /// </summary>
        private void stunButton_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = OpType.RADIO_INHIBIT,
                Argument = ArgType.NO_ARG,
                UnitID = targetID
            };
            Encoder.CreateSingle(pckt);
            SaveGeneratedData();
        }

        /// <summary>
        /// Occurs when the "Revive Target" button is pressed.
        /// </summary>
        private void reviveButton_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = OpType.RADIO_INHIBIT,
                Argument = ArgType.CANCEL_INHIBIT,
                UnitID = targetID
            };
            Encoder.CreateSingle(pckt);
            SaveGeneratedData();
        }

        /// <summary>
        /// Occurs when the "Radio Check" button is pressed.
        /// </summary>
        private void radioCheckButton_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = OpType.RADIO_CHECK,
                Argument = ArgType.RADIO_CHECK,
                UnitID = targetID
            };
            Encoder.CreateSingle(pckt);
            SaveGeneratedData();
        }

        /// <summary>
        /// Occurs when the "Enable Debug Mode" menu item is clicked.
        /// </summary>
        private void enableDebugModeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (enableAdvancedModeToolStripMenuItem.Checked)
            {
                this.MaximumSize = this.MinimumSize = this.Size = new Size(this.Width, 340);
                this.firstGroupBox.Visible = false;
                this.secondGroupBox.Visible = false;
                this.enableAdvancedModeToolStripMenuItem.Checked = false;
            }
            else
            {
                this.MaximumSize = this.MinimumSize = this.Size = new Size(this.Width, 426);
                this.firstGroupBox.Visible = true;
                this.secondGroupBox.Visible = true;
                this.enableAdvancedModeToolStripMenuItem.Checked = true;
            }
        }

        /// <summary>
        /// Occurs when the text in the first op text box is changed.
        /// </summary>
        private void firstOpTextBox_TextChanged(object sender, EventArgs e)
        {
            if (firstOpTextBox.Text.Length < 2)
            {
                firstOpTextBox.ForeColor = Color.Red;
                debugFirstOp = 0x00;
            }
            else
            {
                try
                {
                    firstOpTextBox.ForeColor = Color.Black;
                    debugFirstOp = Convert.ToByte(firstOpTextBox.Text, 16);
                }
                catch (FormatException)
                {
                    firstOpTextBox.ForeColor = Color.Red;
                    debugFirstOp = 0x00;
                }
            }
        }

        /// <summary>
        /// Occurs when the text in the first arg text box is changed.
        /// </summary>
        private void firstArgTextBox_TextChanged(object sender, EventArgs e)
        {
            if (firstArgTextBox.Text.Length < 2)
            {
                firstArgTextBox.ForeColor = Color.Red;
                debugFirstArg = 0x00;
            }
            else
            {
                try
                {
                    firstArgTextBox.ForeColor = Color.Black;
                    debugFirstArg = Convert.ToByte(firstArgTextBox.Text, 16);
                }
                catch (FormatException)
                {
                    firstArgTextBox.ForeColor = Color.Red;
                    debugFirstArg = 0x00;
                }
            }
        }

        /// <summary>
        /// Occurs when the text in the first unit text box is changed.
        /// </summary>
        private void firstUnitIDTextBox_TextChanged(object sender, EventArgs e)
        {
            if (firstUnitIDTextBox.Text.Length < 4)
            {
                firstUnitIDTextBox.ForeColor = Color.Red;
                debugFirstUnitID = 0x0000;
            }
            else
            {
                try
                {
                    firstUnitIDTextBox.ForeColor = Color.Black;
                    debugFirstUnitID = Convert.ToUInt16(firstUnitIDTextBox.Text, 16);
                }
                catch (FormatException)
                {
                    firstUnitIDTextBox.ForeColor = Color.Red;
                    debugFirstUnitID = 0x0000;
                }
            }
        }

        /// <summary>
        /// Occurs when the text in the second op text box is changed.
        /// </summary>
        private void secondOpTextBox_TextChanged(object sender, EventArgs e)
        {
            if (secondOpTextBox.Text.Length < 2)
            {
                secondOpTextBox.ForeColor = Color.Red;
                debugSecondOp = 0x00;
            }
            else
            {
                try
                {
                    secondOpTextBox.ForeColor = Color.Black;
                    debugSecondOp = Convert.ToByte(secondOpTextBox.Text, 16);
                }
                catch (FormatException)
                {
                    secondOpTextBox.ForeColor = Color.Red;
                    debugSecondOp = 0x00;
                }
            }
        }

        /// <summary>
        /// Occurs when the text in the second arg text box is changed.
        /// </summary>
        private void secondArgTextBox_TextChanged(object sender, EventArgs e)
        {
            if (secondArgTextBox.Text.Length < 2)
            {
                secondArgTextBox.ForeColor = Color.Red;
                debugSecondArg = 0x00;
            }
            else
            {
                try
                {
                    secondArgTextBox.ForeColor = Color.Black;
                    debugSecondArg = Convert.ToByte(secondArgTextBox.Text, 16);
                }
                catch (FormatException)
                {
                    secondArgTextBox.ForeColor = Color.Red;
                    debugSecondArg = 0x00;
                }
            }
        }

        /// <summary>
        /// Occurs when the text in the second unit text box is changed.
        /// </summary>
        private void secondUnitIDTextBox_TextChanged(object sender, EventArgs e)
        {
            if (secondUnitIDTextBox.Text.Length < 4)
            {
                secondUnitIDTextBox.ForeColor = Color.Red;
                debugSecondUnitID = 0x0000;
            }
            else
            {
                try
                {
                    secondUnitIDTextBox.ForeColor = Color.Black;
                    debugSecondUnitID = Convert.ToUInt16(secondUnitIDTextBox.Text, 16);
                }
                catch (FormatException)
                {
                    secondUnitIDTextBox.ForeColor = Color.Red;
                    debugSecondUnitID = 0x0000;
                }
            }
        }

        /// <summary>
        /// Occurs when the "Encode Single Packet" button is clicked.
        /// </summary>
        private void mdcDebugSingle_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = debugFirstOp,
                Argument = debugFirstArg,
                UnitID = debugFirstUnitID
            };
            Encoder.CreateSingle(pckt);
            SaveGeneratedData();
        }

        /// <summary>
        /// Occurs when the "Encode Double Packet" button is clicked.
        /// </summary>
        private void mdcDebugDouble_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = debugFirstOp,
                Argument = debugFirstArg,
                UnitID = debugFirstUnitID
            };
            MDCPacket pckt2 = new MDCPacket()
            {
                Operation = debugSecondOp,
                Argument = debugSecondArg,
                UnitID = debugSecondUnitID
            };
            Encoder.CreateDouble(pckt, pckt2);
            SaveGeneratedData();
        }

        /// <summary>
        /// Occurs when the text in the preambles text box is changed.
        /// </summary>
        private void numOfPreamblesTextBox_TextChanged(object sender, EventArgs e)
        {

            try
            {
                numOfPreamblesTextBox.ForeColor = Color.Black;
                int numOfPreambles = Convert.ToInt32(numOfPreamblesTextBox.Text);

                // limit the maximum number of preambles
                if (numOfPreambles > MAX_PREAMBLES_ALLOWED)
                {
                    numOfPreamblesTextBox.Text = string.Empty + MAX_PREAMBLES_ALLOWED;
                    Encoder.NumberOfPreambles = 10;
                }
                else
                    this.Encoder.NumberOfPreambles = numOfPreambles;
            }
            catch (FormatException)
            {
                numOfPreamblesTextBox.ForeColor = Color.Red;
                this.Encoder.NumberOfPreambles = 8;
            }
        }

        /// <summary>
        /// Occurs when the "Call Alert/Page" button is pressed.
        /// </summary>
        private void callAlertButton_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = OpType.DOUBLE_PACKET_TYPE1,
                Argument = ArgType.DOUBLE_PACKET_TO,
                UnitID = targetID
            };
            MDCPacket pckt2 = new MDCPacket()
            {
                Operation = OpType.CALL_ALERT_ACK_EXPECTED,
                Argument = ArgType.CALL_ALERT,
                UnitID = myID
            };
            Encoder.CreateDouble(pckt, pckt2);
            SaveGeneratedData();
        }

        /// <summary>
        /// Occurs when the "SelCall" button is pressed.
        /// </summary>
        private void selectiveCallButton_Click(object sender, EventArgs e)
        {
            // generate MDC packet
            MDCPacket pckt = new MDCPacket()
            {
                Operation = OpType.DOUBLE_PACKET_TYPE1,
                Argument = ArgType.DOUBLE_PACKET_TO,
                UnitID = targetID
            };
            MDCPacket pckt2 = new MDCPacket()
            {
                Operation = OpType.SELECTIVE_CALL_1,
                Argument = ArgType.CALL_ALERT,
                UnitID = myID
            };
            Encoder.CreateDouble(pckt, pckt2);
            SaveGeneratedData();
        }
    } // public partial class MainWindow : Form
} // namespace MDCTool
